#!/usr/bin/perl

package Sympa::Template::Plugin::Sponsoring;
our $VERSION = 0.01;

use strict;
use warnings;
use base qw( Template::Plugin );
use Template::Plugin;
use Sympa::Template::Plugin::Sponsoring::Commons;

# Called as Sympa::Template::Plugin::Sponsoring->load($context)
# by TT2 engine
sub load {
	my ($class, $context) = @_;
	return $class;
}

# Called as Sympa::Template::Plugin::Sponsoring->new($context, @params)
# when using [% USE Sponsoring(param1, param2, ...) %] in TT2 template
sub new {
	my ($class, $context, @params) = @_;
	
	my $stash = $context->stash();
	
	my $self = {
		listname => $stash->get('list'),
			robot => $stash->get('robot'),
		sponsor => $stash->get(['session', 0, 'email', 0]),
		'context' => $context,
		'in' => $stash->get(['plugin', 0, 'Sponsoring', 0]), # Get plugin.Sponsoring.* query vars
		'class' => $class,
	};
	
	eval {
		$self->{root} = Sympa::Template::Plugin::Sponsoring::Commons->new();
	};
	
	if ($@) {
		return $class->error($@);
	}
	
	bless $self, $class;
	
	my $sponsored_lists = $self->{root}->get_sponsored_lists();
	foreach my $sponsor_list (@{$sponsored_lists->{$self->{listname}}{sponsor_lists}}) {
		if ($self->{sponsor_list_object} = Sympa::List->new($sponsor_list)) {
			$self->{admin} = 'yes' if(Sympa::is_listmaster($self->{sponsor_list_object}->{'domain'},$self->{sponsor}));
			$self->{admin} = 'yes' if($self->{sponsor_list_object}->is_admin('owner', $self->{sponsor}));
		} 
	}
	$self->{root}{log}->syslog('info', "[Sponsoring] 3: %s, admin: %s", $self->{sponsor}, $self->{admin});

	unless ($self->{root}->is_list_sponsored($stash->get('list'))) {
		return $class->error(sprintf 'List %s is not a sponsored list',$stash->get('list') );
	}else{
		$self->{list} = $sponsored_lists->{$self->{listname}};
	}
	
	my $godsons = $self->{root}->get_godsons($self->{sponsor},$self->{listname});
	$self->{godson_emails} = $godsons->{godsons_email};
	
	if (!($self->{sponsor} && $self->{root}->is_allowed_sponsor($self->{sponsor},$self->{listname}))) {
		$self->{root}->{log}->syslog('info', "[Sponsoring] 1: %s", $self->{root}->is_allowed_sponsor($self->{sponsor},$self->{listname}));
		$self->{root}->{log}->syslog('info', "[Sponsoring] 1: %s", $self->{sponsor});
		$self->{is_allowed_user} = 0;
	}else{
		$self->{root}->{log}->syslog('info', "[Sponsoring] 2: %s", $self->{sponsor});
		$self->{is_allowed_user} = 1;
	}
		
	return $self;
}

# Godson addition / removal through form
sub handleform {
	my ($self) = @_;
	$self->{in} = $self->{'context'}->stash()->get(['plugin', 0, 'Sponsoring', 0]);
	return 0 unless ($self->{in});
	if ($self->{in} && $self->{in}{lookforsponsor}) {
		return $self->{root}->get_sponsor($self->{in}{godson_email},$self->{listname});
	}
	elsif ($self->{in} && $self->{in}{lookforgodsons}) {
		return $self->{root}->get_godsons($self->{in}{sponsor_email},$self->{listname});
	}
	elsif ($self->{in} && $self->{in}{delete}) {
		foreach my $param (keys %{$self->{in}}) {
			if ($param =~ m{to_delete_(.+)\Z}) {
				$self->{root}->del_godson($self->{in}{$param},$self->{in}{send_mail_to_deleted},$self->{listname},$self->{sponsor});
			}
			my @new_godson_array;
			my $deleted = $self->{in}{$param};
			foreach my $godson (@{$self->{godson_emails}}) {
				unless ($godson =~ m{\A$deleted\Z}) {
					push @new_godson_array, $godson;
				}
			}
			$self->{godson_emails} = \@new_godson_array;
		}
	}
	elsif ($self->{in} && $self->{in}{add}) {
		eval {$self->{root}->add_godson($self->{in}{to_add},$self->{in}{send_mail_to_added},$self->{listname},$self->{sponsor});};
		unless ($@) {
			push @{$self->{godson_emails}}, $self->{in}{to_add};
		}else{
			return $self->{class}->error($@);
		}
	}
	return 1;
}

sub throw {
    my ($self, $error) = @_;
    die (Template::Exception->new('File', $error));
}

# Package must return trueish value
1;
